import { lngLatToMeters } from '@antv/l7-utils';
import earcut from 'earcut';
import { vec3 } from 'gl-matrix';
import getNormals from '../utils/polylineNormal';
import extrudePolygon, { extrude_PolygonNormal, fillPolygon } from './shape/extrude';
import { geometryShape } from './shape/Path';
const GeometryCache = {};
export function PointFillTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates, ...coordinates, ...coordinates, ...coordinates],
    indices: [0, 1, 2, 2, 3, 0],
    size: coordinates.length
  };
}
export function PointExtrudeTriangulation(feature) {
  const {
    shape
  } = feature;
  const {
    positions,
    index,
    normals
  } = getGeometry(shape, false);
  return {
    vertices: positions,
    indices: index,
    normals,
    size: 3
  };
}
export function PointImageTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates],
    indices: [0],
    size: coordinates.length
  };
}
export function LineTriangulation(feature) {
  const {
    coordinates
  } = feature;
  let path = coordinates;

  if (Array.isArray(path[0][0])) {
    path = coordinates[0];
  }

  const line = getNormals(path, false, 0);
  return {
    vertices: line.attrPos,
    indices: line.attrIndex,
    normals: line.normals,
    size: 6
  };
}
export function polygonTriangulation(feature) {
  const {
    coordinates
  } = feature;
  const flattengeo = earcut.flatten(coordinates);
  const {
    vertices,
    dimensions,
    holes
  } = flattengeo;
  return {
    indices: earcut(vertices, holes, dimensions),
    vertices,
    size: dimensions
  };
}
export function PolygonExtrudeTriangulation(feature) {
  const coordinates = feature.coordinates;
  const {
    positions,
    index,
    normals
  } = extrude_PolygonNormal(coordinates, true);
  return {
    vertices: positions,
    indices: index,
    normals,
    size: 3
  };
}
export function HeatmapGridTriangulation(feature) {
  const {
    shape
  } = feature;
  const {
    positions,
    index
  } = getHeatmapGeometry(shape);
  return {
    vertices: positions,
    indices: index,
    size: 3
  };
}
export function RasterImageTriangulation(feature) {
  const coordinates = feature.coordinates;
  const positions = [...coordinates[0], 0, 0, 1, coordinates[1][0], coordinates[0][1], 0, 1, 1, ...coordinates[1], 0, 1, 0, ...coordinates[0], 0, 0, 1, ...coordinates[1], 0, 1, 0, coordinates[0][0], coordinates[1][1], 0, 0, 0];
  const indexs = [0, 1, 2, 3, 4, 5];
  return {
    vertices: positions,
    indices: indexs,
    size: 5
  };
}
export function LineArcTriangulation(feature) {
  const segNum = 30;
  const coordinates = feature.coordinates;
  const positions = [];
  const indexArray = [];

  for (let i = 0; i < segNum; i++) {
    positions.push(i, 1, i, coordinates[0][0], coordinates[0][1], coordinates[1][0], coordinates[1][1], i, -1, i, coordinates[0][0], coordinates[0][1], coordinates[1][0], coordinates[1][1]);

    if (i !== segNum - 1) {
      indexArray.push(...[0, 1, 2, 1, 3, 2].map(v => {
        return i * 2 + v;
      }));
    }
  }

  return {
    vertices: positions,
    indices: indexArray,
    size: 7
  };
}
export function HeatmapTriangulation(feature) {
  const coordinates = feature.coordinates;

  if (coordinates.length === 2) {
    coordinates.push(0);
  }

  const size = feature.size;
  const dir = addDir(-1, 1);
  const dir1 = addDir(1, 1);
  const dir2 = addDir(-1, -1);
  const dir3 = addDir(1, -1);
  const positions = [...coordinates, ...dir, ...coordinates, ...dir2, ...coordinates, ...dir3, ...coordinates, ...dir1];
  const indexArray = [0, 1, 2, 3, 0, 2];
  return {
    vertices: positions,
    indices: indexArray,
    size: 5
  };
}

function getGeometry(shape, needFlat = false) {
  if (GeometryCache && GeometryCache[shape]) {
    return GeometryCache[shape];
  }

  const path = geometryShape[shape] ? geometryShape[shape]() : geometryShape.cylinder();
  const geometry = extrude_PolygonNormal([path], needFlat);
  GeometryCache[shape] = geometry;
  return geometry;
}

function computeVertexNormals(positions, indexArray, dim = 3, needFlat = false) {
  const normals = new Float32Array(positions.length / dim * 3);
  let vA;
  let vB;
  let vC;
  const cb = vec3.create();
  const ab = vec3.create();
  const normal = vec3.create();

  for (let i = 0, li = indexArray.length; i < li; i += 3) {
    vA = indexArray[i + 0] * 3;
    vB = indexArray[i + 1] * 3;
    vC = indexArray[i + 2] * 3;
    let p1 = [positions[vA], positions[vA + 1]];
    let p2 = [positions[vB], positions[vB + 1]];
    let p3 = [positions[vC], positions[vC + 1]];

    if (needFlat) {
      p1 = lngLatToMeters(p1);
      p2 = lngLatToMeters(p2);
      p3 = lngLatToMeters(p3);
    }

    const [ax, ay] = p1;
    const pA = vec3.fromValues(ax, ay, positions[vA + 2]);
    const [bx, by] = p2;
    const pB = vec3.fromValues(bx, by, positions[vB + 2]);
    const [cx, cy] = p3;
    const pC = vec3.fromValues(cx, cy, positions[vC + 2]);
    vec3.sub(cb, pC, pB);
    vec3.sub(ab, pA, pB);
    vec3.cross(normal, cb, ab);
    normals[vA] += cb[0];
    normals[vA + 1] += cb[1];
    normals[vA + 2] += cb[2];
    normals[vB] += cb[0];
    normals[vB + 1] += cb[1];
    normals[vB + 2] += cb[2];
    normals[vC] += cb[0];
    normals[vC + 1] += cb[1];
    normals[vC + 2] += cb[2];
  }

  normalizeNormals(normals);
  return normals;
}

function normalizeNormals(normals) {
  for (let i = 0, li = normals.length; i < li; i += 3) {
    const normal = vec3.fromValues(normals[i], normals[i + 1], normals[i + 2]);
    const newNormal = vec3.create();
    vec3.normalize(newNormal, normal);
    normals.set(newNormal, i);
  }
}

function checkIsClosed(points) {
  const p1 = points[0][0];
  const p2 = points[0][points[0].length - 1];
  return p1[0] === p2[0] && p1[1] === p2[1];
}

function getHeatmapGeometry(shape) {
  const shape3d = ['cylinder', 'triangleColumn', 'hexagonColumn', 'squareColumn'];
  const path = geometryShape[shape] ? geometryShape[shape]() : geometryShape.circle();
  const geometry = shape3d.indexOf(shape) === -1 ? fillPolygon([path]) : extrudePolygon([path]);
  return geometry;
}

function addDir(dirX, dirY) {
  const x = (dirX + 1) / 2;
  const y = (dirY + 1) / 2;
  return [x, y];
}
//# sourceMappingURL=triangulation.js.map