import { gl } from '@antv/l7-core';
import BaseLayer from '../core/BaseLayer';
import { generateColorRamp } from '../utils/color';
import { RasterTriangulation } from './buffers/triangulation';
const rasterFrag = "varying vec4 v_color;\nuniform float u_opacity: 1.0;\n#define PI 3.141592653589793\n\nvoid main() {\n   gl_FragColor = v_color;\n   gl_FragColor.a *= u_opacity;\n\n}\n";
const rasterVert = "precision highp float;\nattribute vec3 a_Position;\nuniform mat4 u_ModelMatrix;\nuniform vec4 u_extent;\nuniform sampler2D u_texture;\nuniform sampler2D u_colorTexture;\nuniform float u_min;\nuniform float u_max;\nuniform float u_width;\nuniform float u_height;\nuniform float u_heightRatio;\n\nvarying vec2 v_texCoord;\nvarying vec4 v_color;\n\n#pragma include \"projection\"\nvoid main() {\n  vec2 uv = a_Position.xy / vec2(u_width, u_height);\n  vec2 minxy =  project_position(vec4(u_extent.xy, 0, 1.0)).xy;\n  vec2 maxxy =  project_position(vec4(u_extent.zw, 0, 1.0)).xy;\n  float value = texture2D(u_texture, vec2(uv.x,1.0 - uv.y)).x;\n  vec2 step = (maxxy - minxy) / vec2(u_width, u_height);\n  vec2 pos = minxy + vec2(a_Position.x, a_Position.y ) * step;\n  //  v_texCoord = a_Uv;\n  value = clamp(value,u_min,u_max);\n  float value1 =  (value - u_min) / (u_max -u_min);\n  vec2 ramp_pos = vec2(\n        fract(16.0 * (1.0 - value1)),\n        floor(16.0 * (1.0 - value1)) / 16.0);\n  v_color = texture2D(u_colorTexture,ramp_pos);\n\n  // if(uv.x > 1.0 || uv.y > 1.0) {\n  //   v_color = vec4(0.);\n  // }\n\n  //  vec2 range = u_extent.zw - u_extent.xy;\n  //  vec4 project_pos = project_position(vec4(pos, 0, 1.0));\n   gl_Position = project_common_position_to_clipspace(vec4(pos.xy, project_scale(value) * u_heightRatio, 1.0));\n\n}\n";
export default class RasterLayer extends BaseLayer {
  constructor(...args) {
    super(...args);
    this.name = 'e';
    this.texture = void 0;
    this.colorTexture = void 0;
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity,
      heightRatio = 10
    } = this.getLayerConfig();
    const parserDataItem = this.getSource().data.dataArray[0];
    const {
      coordinates,
      width,
      height,
      min,
      max
    } = parserDataItem;
    this.models.forEach(model => model.draw({
      uniforms: {
        u_opacity: opacity || 1,
        u_texture: this.texture,
        u_min: min,
        u_width: width,
        u_height: height,
        u_max: max,
        u_heightRatio: heightRatio,
        u_colorTexture: this.colorTexture,
        u_extent: [...coordinates[0], ...coordinates[1]]
      }
    }));
    return this;
  }

  buildModels() {
    const parserDataItem = this.getSource().data.dataArray[0];
    const {
      createTexture2D
    } = this.rendererService;
    this.texture = createTexture2D({
      data: parserDataItem.data,
      width: parserDataItem.width,
      height: parserDataItem.height,
      format: gl.LUMINANCE,
      type: gl.FLOAT,
      aniso: 4
    });
    const {
      rampColors
    } = this.getLayerConfig();
    const imageData = generateColorRamp(rampColors);
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      flipY: true
    });
    this.models = [this.buildRasterModel()];
  }

  buildRasterModel() {
    const source = this.getSource();
    const sourceFeature = source.data.dataArray[0];
    const triangulation = RasterTriangulation(sourceFeature);
    this.shaderModuleService.registerModule('raster', {
      vs: rasterVert,
      fs: rasterFrag
    });
    const {
      vs,
      fs,
      uniforms
    } = this.shaderModuleService.getModule('raster');
    const {
      createAttribute,
      createElements,
      createBuffer,
      createModel
    } = this.rendererService;
    return createModel({
      vs,
      fs,
      attributes: {
        a_Position: createAttribute({
          buffer: createBuffer({
            data: triangulation.vertices,
            type: gl.FLOAT
          }),
          size: 2
        })
      },
      primitive: gl.TRIANGLES,
      uniforms: { ...uniforms
      },
      depth: {
        enable: true
      },
      elements: createElements({
        data: triangulation.indices,
        type: gl.UNSIGNED_INT,
        count: triangulation.indices.length
      })
    });
  }

}
//# sourceMappingURL=raster.js.map